/*
	etrnquery.c -- simple Batched SMTP transfer initiator
	(C) Wouter Coene, 1999

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, write to the Free Software
	Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define WANT_UNISTD
#define WANT_SYS_TYPES
#include "config-wrap.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "ecmdline.h"
#include "omta.h"
#include "readconf.h"
#include "canon-host.h"
#include "lib.h"
#include "log.h"

char *last_message=NULL;

/***
 * etrn_expect
 * Read a response from the SMTP server and check if it's the
 * response we're expecting.
 */
int etrn_expect(char *str, FILE *fd)
{
	char *s;

	if (last_message==NULL)
		last_message=stralloc(1024);

	s=stralloc(1024);
	s=fgets(s,1024,fd);
	if (s!=NULL) {
		while ((*(s+strlen(s)-1)=='\n') || (*(s+strlen(s)-1)=='\r'))
			*(s+strlen(s)-1)='\0';
		last_message=strncpy(last_message,s,1024);

		if (strstr(s,str)!=s)
			return FALSE;
		else
			return TRUE;
	} else
		return FALSE;
}

/***
 * etrn_connect
 * Open a connection to the specified SMTP server.
 */
FILE *etrn_connect(const char *server, int port)
{
	struct hostent *h;
	struct sockaddr_in sock;
	int sid;
	FILE *fd;
	char *n;
	char *c;

	if ((sid=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP))==-1) {
		perror("socket");
		return NULL;
	}

	h=gethostbyname(server);
	if (h==NULL) {
		herror("gethostbyname");
		return NULL;
	}

	sock.sin_family=AF_INET;
	sock.sin_port=htons(port);
	memcpy(&sock.sin_addr,h->h_addr,h->h_length);
	
	if (connect(sid,(struct sockaddr *) &sock,sizeof(sock))==-1) {
		perror("connect");
		return NULL;
	}

	fd=fdopen(sid,"a+");

	if (!etrn_expect("220",fd))
		return NULL;

	/*
	 * EHLO name is user-configurable, with the default being the
	 * host's canonical name.
	 */
	if (conf.host_name!=NULL) {
	        c=conf.host_name;
	} else {
		n=stralloc(MAX_LINE_LEN);
	        gethostname(n,MAX_LINE_LEN-1);
	        c=canon_host(n);
	        free(n);
	}
	if (c==NULL) {
		c=strdup("localhost");
	}
	fprintf(fd,"EHLO %s\r\n",c); /* use ESMTP */
	
	return fd;
}

/***
 * mode_query
 * Do the BSMTP-ETRN query.
 */
int mode_query()
{
	int i, done, does_etrn, ok;
	FILE *fd;
	char *s, *p;

	log_enter();

	fd=NULL; /* to make gcc stop whining */
	ok=FALSE;
	do {
		if (conf.local_count==0) {
			printf("No local hostnames defined in your configuration file\n");
			break;
		}

		if ((fd=etrn_connect(conf.smtp_server,conf.smtp_port))==NULL)
			break;
		printf("Connected to server %s port %d\n",conf.smtp_server,conf.smtp_port);

		p=stralloc(1024);

		done=does_etrn=FALSE;
		while (!done) {
			s=fgets(p,1024,fd);
			if ((s==NULL) || feof(fd)) {
				printf("Remote host closed connection\n");
				break;
			}

			s=trim(s,"\n\r");
			if (strstr(s,"250")!=s) {
				printf("Remote host does not support ESMTP\n");
				done=FALSE;
				break;
			}

			if (strstr(s,"250-")!=s)
				done=TRUE;

			s+=4;
			if (strcmp(s,"ETRN")==0)
				does_etrn=TRUE;
		}

		if (!done)
			break;

		if (!does_etrn) {
			printf("Remote host does not support ETRN\n");
			log_leave();
			return FALSE;
		}

		for (i=0; i<conf.local_count; i++) {
			fprintf(fd,"ETRN %s\r\n",conf.local[i]);
			done=FALSE;
			while (!done) {
				s=fgets(p,1024,fd);
				if ((s==NULL) || feof(fd)) {
					printf("Remote host closed connection\n");
					break;
				}

				s=trim(s,"\n\r");
				if (*(s+3)!='-')
					done=TRUE;
				if (*s=='5') {
					printf("That went wrong, quitting\n");
					break;
				}
				s+=4;
				printf("%s: %s\n",conf.local[i],s);
			}
		}

		printf("%d requests sent, closing connection\n",conf.local_count);
		
		ok=TRUE;
	} while (1==0);

	if (fd!=NULL) {
		fprintf(fd,"QUIT\r\n");
		if (!etrn_expect("221",fd)) {
			if (shutdown(fileno(fd),0)==-1)
				perror("shutdown");
		}

		fclose(fd);
	}

	log_leave();
	return ok;
}

/***
 * defaults
 * Write compile-time defaults to standard output.
 */
void defaults()
{
	short_version();
	printf("Compile-time defaults:\n\n");

	printf("Global configuration file:\t\t" CONFIG_GLOBAL "\n");
	printf("Local (per-user) configuration file:\t" CONFIG_LOCAL "\n");
}

/***
 * run_mode
 * Do something based on the specified operation mode.
 */
int run_mode(int mode)
{
	int ok;

	ok=FALSE;
	switch (mode) {
		case MODE_QUERY:
			ok=mode_query();
			break;

		case MODE_HELP:
			help();
			ok=TRUE;
			break;

		case MODE_SHORTHELP:
			short_help();
			break;

		case MODE_VERSION:
			version();
			ok=TRUE;
			break;

		case MODE_DEFAULTS:
			defaults();
			ok=TRUE;
			break;
	}

	return ok;
}

/***
 * main
 * The startup routine.
 */
int main(int argc, char *argv[])
{
	int mode;

	mode=cmdline(argc,argv);
	if (mode<MODE_NOMODE) {
		config(TRUE);
	}

	return (run_mode(mode)? 0:1);
}

/* end etrnquery.c */
